/*
 * @file xaxivdma_example_selftest.c
 *
 * <pre>
 * MODIFICATION HISTORY:
 *
 *
 */

/***************************** Include Files *********************************/
#include "main.h"

/************************** Variable Definitions *****************************/
/*
 * Device instance definitions
 */
XGpio          Gpio;          /* The Instance of the GPIO 0 Driver */
XGpio          Gpio_1;        /* The Instance of the GPIO 1 Driver */


/*
 * Frame pointer locations per Read/Write VDMA
 */
volatile u32	frame_read[6]  = {FRAME_R_BASE_1,FRAME_R_BASE_2,FRAME_R_BASE_3,FRAME_R_BASE_1,FRAME_R_BASE_2,FRAME_R_BASE_3};
volatile u32	frame_write[6] = {FRAME_W_BASE_1,FRAME_W_BASE_2,FRAME_W_BASE_3,FRAME_W_BASE_1,FRAME_W_BASE_2,FRAME_W_BASE_3};
volatile u32	frame_cnt;

/*****************************************************************************/
/**
* The entry point for the WIDI project. It initializes and configures 
* the devices then starts the 60Hz timer and reports the execution status.
*
* @param    None.
*
* @return
*     - XST_SUCCESS if successful
*     - XST_FAILURE if fails.
*
* @note     None.
*
******************************************************************************/
int main()
{
   int Status;

   frame_cnt = 0;

   xil_printf("\r\n--- Entering main() --- \r\n");

   Status = TxConfig();
   if (Status != XST_SUCCESS) {
	  xil_printf("TX Configuration Failed\r\n");
	  return XST_FAILURE;
   }

   //Flush cache
   Xil_DCacheFlush();


   //Start the timer counter
   XTtcPs_Start(&Timer);

   xil_printf("--- Exiting main() --- \r\n");


   return XST_SUCCESS;

}

int TxConfig()
{
   int Status;
   u32 dma_MMCtrReg;
   u32 dma_SCtrReg;

   //Initialize GPIO for read frame pointer input
   Status = XGpio_Initialize(&Gpio, GPIO_DEV_ID);
   if (Status != XST_SUCCESS) {
	  xil_printf("Gpio Initialization Failed\r\n");
	  return XST_FAILURE;
   }

   //Initialize GPIO_1 for write frame pointer input
   Status = XGpio_Initialize(&Gpio_1, GPIO_DEV_1_ID);
   if (Status != XST_SUCCESS) {
	  xil_printf("Gpio Initialization Failed\r\n");
	  return XST_FAILURE;
   }

   //Initialize and Test Write VDMA
   Status = AxiVDMASelfTestExample(&AxiVdma,DMA_DEV_ID);

   if (Status != XST_SUCCESS) {

	  xil_printf("AxiVDMASelfTest Example Failed\r\n");
	  return XST_FAILURE;
   }

   xil_printf("Successfully ran Write AxiVDMASelfTest Example\r\n");

   //Initialize and Test Read VDMA
   Status = AxiVDMASelfTestExample(&AxiVdma_R,DMA_DEV_1_ID);

   if (Status != XST_SUCCESS) {

	  xil_printf("AxiVDMASelfTest Example Failed\r\n");
	  return XST_FAILURE;
   }

   xil_printf("Successfully ran Write AxiVDMASelfTest Example\r\n");

   //Configure VDMA Write
   Status = configureWrite(H_RES, V_RES);

   if (Status != (XST_SUCCESS)) {
	  xil_printf("VDMA Config Write Failed.\r\n");
	  return (XST_FAILURE);
   }

   //Enable VDMA Write
   Status = XAxiVdma_DmaStart(&AxiVdma, XAXIVDMA_WRITE);

   if (Status != (XST_SUCCESS)) {
	  xil_printf("VDMA Enable Write Failed.\r\n");
	  return (XST_FAILURE);
   }

   //Configure VDMA Read
   Status = configureRead(H_RES, V_RES);

   if (Status != (XST_SUCCESS)) {
	  xil_printf("VDMA Configure Read Failed.\r\n");
	  return (XST_FAILURE);
   }

   //Enable VDMA Read
   Status = XAxiVdma_DmaStart(&AxiVdma_R, XAXIVDMA_READ);

   if (Status != (XST_SUCCESS)) {
	  xil_printf("VDMA Enable Read Failed.\r\n");
	  return (XST_FAILURE);
   }

   Status = TimerInit(&Timer, &Intc, TIMER_DEV_ID, TIMER_IRPT);

   if (Status != (XST_SUCCESS)) {
	  xil_printf("Timer Init Failed.\r\n");
	  return (XST_FAILURE);
   }

   dma_MMCtrReg = XAxiVdma_ReadReg((AxiVdma).BaseAddr,
			   (XAXIVDMA_RX_OFFSET));
   xil_printf("DMA Memory Control Register: 0x%08x\r\n", dma_MMCtrReg);

   dma_SCtrReg = XAxiVdma_ReadReg((AxiVdma_R).BaseAddr,
			   (XAXIVDMA_TX_OFFSET));
   xil_printf("DMA Stream Control Register: 0x%08x\r\n", dma_SCtrReg);


   return XST_SUCCESS;
}

/*****************************************************************************/
/**
* This function handles the timer interrupt function. It copys memory from
* VDMA write channel DDR memory to the VDMA read channel DDR memory block
* 
*
* @param    None.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void IRQHanlder(void *CallBackRef){
   u32 *r_addr;
   u32 *w_addr;
   u32 g_code_rd;
   u32 g_code_wr;
   u32 n_rd;
   u32 n_wr;

   u32 StatusEvent;

	/*
	 * Read the interrupt status, then write it back to clear the interrupt.
	 */
   StatusEvent = XTtcPs_GetInterruptStatus((XTtcPs *)CallBackRef);
   XTtcPs_ClearInterruptStatus((XTtcPs *)CallBackRef, StatusEvent);

   frame_cnt++;

   //Copy every 6 frames to TX Frame Buffer
   /* Take this block out of IF statement for HDMI passthru mode */
   if (frame_cnt == FRAME_CNT_10HZ) {

	   frame_cnt = 0;

	   //Get gray code for Write VDMA current frame pointer
	   g_code_wr = XGpio_DiscreteRead(&Gpio_1,PNTR_CHANNEL);
	   g_code_rd = XGpio_DiscreteRead(&Gpio,PNTR_CHANNEL);

	   //Convert gray code to decimal
	   n_wr = grayCode(g_code_wr) - 1;
	   n_rd = grayCode(g_code_rd) - 1;

	   //Wrap to start frame if at last frame
	   if (n_rd == 0) {
		   n_rd = 6;
	   }

	   if (n_wr == 0) {
		   n_wr = 6;
	   }

	   //Point to previous frame to frame pointer
	   r_addr = (u32*)frame_read[n_rd - 1];
	   
      /* Comment out this line for HDMI passthru mode */
      w_addr = testframe;

      /* Uncomment this line for HDMI passthru mode */
      //w_addr = (u32*)frame_read[n_wr - 1];


	   //Copy frame to network memory
	   Xil_DCacheInvalidateRange((u32)r_addr,FRAME_WORDS_X3);
	   Xil_MemCpy(r_addr,w_addr,FRAME_WORDS_X3);
	   Xil_DCacheFlushRange((u32)r_addr,FRAME_WORDS_X3);
   }
   xil_printf("IRQ\r\n");

}


/*****************************************************************************/
/**
* This function performs a memory read and prints the results.
*
* @param    *Addr: start of address to be read
*           Words: 32 bit length of 32 bit words to read
*
* @return
*     - XST_SUCCESS if channel reset is successful
*     - XST_FAILURE if channel reset fails.
*
* @note     None.
*
******************************************************************************/
void ReadMem(u32 *Addr, u32 Words)
{
   u32 J = 6;
   u32 I;
   u32 WordMem;

   /*
    * Read third frame buffer
    */
   for (I = 0U; I < Words; I++) {
       WordMem = *(Addr+I);
       xil_printf("0x%08x | ", WordMem);
       if ((I % J) == 0) {
          xil_printf("\r\n");
       }
   }
}

/*****************************************************************************/
/**
* This function performs a memory wirte.
*
* @param    *Addr: start of address to write to
*           Words: 32 bit length of 32 bit words to write
*           Data: 32 bit data value to write
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void WriteMem(u32 *Addr, u32 Words, u32 Data)
{
   u32 I;
   u32 Val;
   /*
    * variable initialization
    */
   Val = Data;

   for (I = 0U; I < Words; I++) {
      *(Addr+I) = Val;
   }

}

/*****************************************************************************/
/**
* This function converts gray codes to decimal values.
*
* @param    n: gray code
*
* @return   None.
*
* @note     Integer decimal value
*
******************************************************************************/
int grayCode(u32 n)
{
    int inv = 0; 
  
    // Taking xor until n becomes zero 
    for (; n; n = n >> 1) 
        inv ^= n; 
  
    return inv;
}
